<?php

class appWrapProductForGiftPluginListener
{
    public static function generateStProduct(sfEvent $event)
    {
        /**
         * @var stAdminGenerator
         */
        $generator = $event->getSubject();        

        $generator->insertParameterAfter('edit.display.NONE[active]', ['app_wrap_product_for_gift_is_excluded']);
        $generator->setValueForParameter('edit.fields.app_wrap_product_for_gift_is_excluded', [
            'name' => 'Wyklucz z "Zapakuj jako prezent"',
            'i18n' => 'appWrapProductForGiftBackend',
        ]); 
    }

    public static function smartySlotAppend(sfEvent $event, $components)
    {
        if (!appWrapProductForGiftHelper::getInstance()->isEnabled())
        {
            return $components;
        }

        if ($event['slot'] == 'basket-after-item-row')
        {
            $components[] = $event->getSubject()->createComponent('appWrapProductForGiftFrontend', 'renderBasketItemRow');
        }

        if ($event['slot'] == 'basket-widget-after-item-row')
        {
            $components[] = $event->getSubject()->createComponent('appWrapProductForGiftFrontend', 'renderBasketWidgetItemRow');
        }

        if ($event['slot'] == 'order-confirm-after-item-row')
        {
            $components[] = $event->getSubject()->createComponent('appWrapProductForGiftFrontend', 'renderOrderItemRow');
        }

        if ($event['slot'] == 'basket-after-items')
        {
            $components[] = $event->getSubject()->createComponent('appWrapProductForGiftFrontend', 'renderBasketModal');
        }

        return $components;      
    }

    public static function addAssets(sfEvent $event)
    {
        if (!appWrapProductForGiftHelper::getInstance()->isEnabled())
        {
            return;
        }

        /**
         * @var stActions $controller
         */
        $controller = $event->getSubject();

        /**
         * @var sfWebResponse
         */
        $response = $controller->getResponse();

        if ($controller->getModuleName() == 'stBasket' || $controller->getModuleName() == 'stOrder' && $controller->getActionName() == 'confirm')
        {
            $response->addJavascript('/plugins/appWrapProductForGiftPlugin/js/script.js');
        }

        $response->addStylesheet('/plugins/appWrapProductForGiftPlugin/css/style.css', 'last');
    }

    public static function filterBasketTotalAmount(sfEvent $event, $totalAmount)
    {
        if (!appWrapProductForGiftHelper::getInstance()->isEnabled())
        {
            return $totalAmount;
        }

        /**
         * @var BasketProduct $item
         */
        $item = $event->getSubject();

        $wrapGiftItem = appWrapProductForGiftHelper::getInstance()->get($item);

        if (null !== $wrapGiftItem)
        {
            $prevValue = $wrapGiftItem->getVatEu();
            
            if (!$event['with_vat_eu'])
            {
               $item->setVatEu(false);
            }

            $totalAmount += $wrapGiftItem->getTotalAmount($event['with_tax'], $event['with_currency'], false);

            $wrapGiftItem->setVatEu($prevValue);
        }

        return $totalAmount;
    }

    public static function filterBasketProductMaxQuantity(sfEvent $event, $maxQuantity)
    {
        if (!appWrapProductForGiftHelper::getInstance()->isEnabled())
        {
            return $maxQuantity;
        }

        /**
         * @var BasketProduct
         */
        $item = $event->getSubject();

        $wrapGiftProduct = appWrapProductForGiftHelper::getInstance()->getProduct($item);

        if (null !== $wrapGiftProduct && $wrapGiftProduct->getStock() < $maxQuantity)
        {
            $maxQuantity = $wrapGiftProduct->getStock();
        }

        return $maxQuantity;
    }

    public static function postMirrorOrderProduct(sfEvent $event)
    {
        if (!appWrapProductForGiftHelper::getInstance()->isEnabled())
        {
            return;
        }

        $i18n = sfContext::getInstance()->getI18N();

        /**
         * @var Order
         */
        $order = $event->getSubject();

        /**
         * @var BasketProduct
         */
        $basketProduct = $event['basket_product'];

        $wrapGiftBasketProduct = appWrapProductForGiftHelper::getInstance()->get($basketProduct);

        if (null !== $wrapGiftBasketProduct)
        {
            $orderProduct = OrderProductPeer::createFromBasketProduct($wrapGiftBasketProduct, $event['tax'], $event['guard_user']);

            $orderProduct->setName($i18n->__('Zapakuj jako prezent') . ' - ' . $orderProduct->getName()); 

            if (!empty($basketProduct->getAppWrapProductForGiftMessage()))
            {
                $orderProduct->addPriceModifier(null, null, null, null, [
                    'field' => 'Bilecik',
                    'label' => $basketProduct->getAppWrapProductForGiftMessage(),
                    'hide' => ['invoice'],
                ]);
            }
            
            $order->addOrderProduct($orderProduct);
        }
    }
}